import os

# read in the corpus in the given directory
def read_data(directory):

  files_content = {}

  for filename in filter(lambda p: p.endswith("txt"), os.listdir(directory)):
      filepath = os.path.join(directory, filename)
      with open(filepath, mode='r') as f:
          files_content[filename] = f.read().lower()
          
  return files_content

# calculate the document frequency,
# i.e., the number of documents containing the term
def df(data, term):
  s = 0
  for d in data.values():
    if term in d:
      s += 1
  return s

def printResults(results):
  for r in sorted(results.keys(), key=lambda f : results[f], reverse=True):
    print(f"  {r}: {results[r]}")

mode = 0

data = read_data("data")

while(True):
  print("====================")
  query = input("Query: ")
  query = query.lower()
  # some meta commands
  if query.startswith("."):
    if query == ".exit":
      exit(0)
    elif query.startswith(".mode"):
      modelist = query.split(" ")
      if len(modelist) == 1:
        print(f"mode: {mode}")
      elif len(modelist) == 2:
        mode = int(modelist[1])
      continue
  
  if mode == 0:
    # is the query contained in the document?
    results = { f : int(query in data[f]) for f in data }
    printResults(results)
  elif mode == 1:
    # How often is the query contained in the document?
    results = { f : data[f].count(query) for f in data }
    printResults(results)
  elif mode == 2:
    # Split the query into tokens, count each token separately 
    # and add them up
    ql = query.split(" ")
    results = { f : 0 for f in data }
    for qw in ql:
      for f in data:
        results[f] += data[f].count(qw)
    printResults(results)
  elif mode == 3:
    # tfidf, with absolute tf values
    ql = query.split(" ")
    results = { f : 0 for f in data }
    for qw in ql:
      for f in data:
        results[f] += data[f].count(qw) / df(data, qw)
    printResults(results)
  elif mode == 4:
    # tfidf, with relative tf values
    ql = query.split(" ")
    results = { f : 0 for f in data }
    for qw in ql:
      for f in data:
        results[f] += (data[f].count(qw)/len(data[f])) / df(data, qw)
    printResults(results)
